/* Our Color Scheme */
$red-grad-top: #CC271E;
$red-grad-bottom: #A71E25;
$red-logo-border: #7C2424;

$dark-grey: #EEE;
$darker-grey: #DDD;

/* Our Typography */
$heading-font: 'Book Antiqua', 'Georgia', serif;

/* Our Look & Feel */
@mixin gradient {
	background-image: linear-gradient(bottom, $grad-bottom 25%, $grad-top 100%);
	background-image: -o-linear-gradient(bottom, $grad-bottom 25%, $grad-top 100%);
	background-image: -moz-linear-gradient(bottom, $grad-bottom 25%, $grad-top 100%);
	background-image: -webkit-linear-gradient(bottom, $grad-bottom 25%, $grad-top 100%);
	background-image: -ms-linear-gradient(bottom, $grad-bottom 25%, $grad-top 100%);
	background-image: -webkit-gradient(
		linear,
		left bottom,
		left top,
		color-stop(0.25, $grad-bottom),
		color-stop(1, $grad-top)
	);

}
@mixin gradient-shift {
	background-image: linear-gradient(bottom, $grad-bottom 55%, $grad-top 100%);
	background-image: -o-linear-gradient(bottom, $grad-bottom 55%, $grad-top 100%);
	background-image: -moz-linear-gradient(bottom, $grad-bottom 55%, $grad-top 100%);
	background-image: -webkit-linear-gradient(bottom, $grad-bottom 55%, $grad-top 100%);
	background-image: -ms-linear-gradient(bottom, $grad-bottom 55%, $grad-top 100%);
	background-image: -webkit-gradient(
		linear,
		left bottom,
		left top,
		color-stop(0.55, $grad-bottom),
		color-stop(1, $grad-top)
	);
}
@mixin box-shadow {
	box-shadow: 0px 2px 4px 0px #CCC;
}
@mixin rounded-border {
	$radius: 2px;

	border-radius: $radius;
	-moz-border-radius: $radius;
	-webkit-border-radius: $radius;
}
@mixin bg-color-transition {
	$time: 0.2s;

	-webkit-transition: background-color $time ease-in;
	-moz-transition: background-color $time ease-in;
	-o-transition: background-color $time ease-in;
	transition: background-color $time ease-in;
}

/* Functional vars */
$grad-top: $red-grad-top;
$grad-bottom: $red-grad-bottom;
$logo-border: $red-logo-border;
$dark-neutral: $dark-grey;
$darker-neutral: $darker-grey;

/* Override Bootstrap Styles*/
body {
	overflow-x: hidden;
}
.container {
	margin: 0;
	max-width: none;
}
h1, h2, h3, h4, h5, h6 {
	font-family: $heading-font;
}
a {
	color: $grad-bottom;
	text-decoration: none;
	border-bottom: 1px dotted $grad-top;

	&:hover {
		color: $grad-bottom;
		text-decoration: none;
	}
}
.btn {
	@include gradient;
	@include rounded-border;
	@include box-shadow;

	border: 1px solid $logo-border;
	color: #FDFDFD;
	font-weight: normal;
	text-shadow: 0px 1px 1px $logo-border;

	&:hover {
		color: #FFF;
		@include gradient-shift;
	}
}

/* General Styles */
strong.error {
	color: $grad-top;
}
.side-menu {
	padding: {
		left: 0;
		right: 0;
	}
	margin-right: -1px;
	position: relative;
	z-index: 1;

	ul {
		list-style: none;
		padding-left: 0;

		li {
			padding: 20px;
			border-bottom: 1px solid #DDD;
			background-color: #FFF;
		}

		li.active {
			font-weight: bold;
			color: $grad-top;
			box-shadow: -7px 0px 7px 0px #CCC;
		}
	}
}
.main {
	border-left: 1px solid #DDD;
}

/* Header Styles */
#header {
	box-shadow: 0px 2px 4px 0px #CCC;
	@include gradient;

	/* So menu shadow doesn't go over the header*/
	position: relative;
	z-index: 2;
}

.logo-container {
	img {
		padding: 10px;
		border-right: 1px solid $logo-border;
	}
}

/* Left Main Menu Styles */

/* OCR-specific Styles */
.scan-dir-sect {
	text-align: center;

	h4 {
		/*color: $grad-bottom;*/
		font-weight: bold;
	}
}
.helper-links {
	display: block;
	font-size: 12px;

	margin: {
		right: 2px;
		top: 10px;
		left: 10px;
	}

	a {
		@include rounded-border;
		@include bg-color-transition;

		text-decoration: none;
		border: 0;
		float: right;
		margin-right: 5px;
		display: inline-block;
		color: #222;
		background-color: $dark-neutral;

		padding: {
			top: 8px;
			bottom: 5px;
			left: 10px;
			right: 10px;
		}

		&:hover {
			@include bg-color-transition;
			background-color: $darker-neutral;
		}

		.glyphicon {
			margin-right: 5px;
		}
	}
}
#step-progress {
	width: 100%;
	height: 5px;
	margin-bottom: 20px;

	.step-bar {
		width: 18%;
		height: 5px;
		background-color: #DDD;
		display: inline-block;
		-webkit-transition: background-color 0.2s linear;
		-moz-transition: background-color 0.2s linear;
		-o-transition: background-color 0.2s linear;
		transition: background-color 0.2s linear;

		&:hover {
			background-color: #BBB;
			-webkit-transition: background-color 0.2s linear;
			-moz-transition: background-color 0.2s linear;
			-o-transition: background-color 0.2s linear;
			transition: background-color 0.2s linear;
		}
		&.complete {
			background-color: $grad-top;
			-webkit-transition: background-color 0.2s linear;
			-moz-transition: background-color 0.2s linear;
			-o-transition: background-color 0.2s linear;
			transition: background-color 0.2s linear;

			&:hover {
				background-color: $grad-bottom;
				-webkit-transition: background-color 0.2s linear;
				-moz-transition: background-color 0.2s linear;
				-o-transition: background-color 0.2s linear;
				transition: background-color 0.2s linear;

			}
		}
	}
}
#upload-area {
	@include rounded-border;

	min-height: 250px;
	overflow: hidden;
	width: 80%;
	border: 2px dashed #DDD;
	text-align: center;
	margin: {
		left: auto;
		right: auto;
		top: 20px;
		bottom: 20px;
	}

	input {
		@include rounded-border;

		background-color: #EEE;
		padding: 10px;
		margin: {
			left: auto;
			right: auto;
		}
	}

	.message {
		display: inline-block;
		font-size: 24px;
		color: #AAA;
		margin: {
			left: auto;
			right: auto;
			top: 50px;
			bottom: 50px;
		}
	}
}
.pane {
	.btn {
		margin: {
			right: 20px;
			left: 20px;
		}
	}
	.pane-footer {
		float: right;
		margin: 10px;

		a {
			margin-right: 3px;
		}
	}
}
